@@ -10,7 +10,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#ifdef HAVE_LOCALE_H
+#if HAVE_LOCALE_H
 #include <locale.h>
 #endif
 
@@ -48,7 +48,7 @@ extern "C"
 
 
 const char versionOutput[] =
-"FrobTADS " PACKAGE_VERSION "\n"
+"FrobTADS " FROB_VERSION "\n"
 "TADS 2 virtual machive v" TADS_RUNTIME_VERSION "\n"
 "TADS 3 virtual machine v" T3VM_VSN_STRING " (" T3VM_IDENTIFICATION ")\n"
 "FrobTADS copyright (C) 2009 Nikos Chantziaras.\n"
@@ -75,7 +75,9 @@ const char helpOutput[] =
 "  -d, --no-chdir       Don't change the current directory\n"
 "  -S, --no-seed-rand   Do not seed the random number generator\n"
 "  -s, --safety-level   File I/O safety level (default is 2)\n"
+#ifdef TADSNET
 "  -N, --net-safety-level Network safety level\n"
+#endif
 "  -e, --scroll-buffer  Size of the scroll-back buffer. Must be between 8 and\n"
 "                       8192kB (default is 512kB)\n"
 "  -r, --restore        Load a saved game position\n"
@@ -104,7 +106,7 @@ const char helpOutput[] =
 "option that takes an argument is given multiple times the last value is used.\n"
 "Options may be abbreviated: --no-scr will be recognized as --no-scrolling.\n"
 "\n"
-"Report bugs to <" PACKAGE_BUGREPORT ">.\n"
+"Report bugs to <" FROB_BUG_REPORT_EMAIL ">.\n"
 "Please include the output of the --version option with the report.";
 
 
@@ -115,6 +117,7 @@ create_netconfig( TadsNetConfig*& c, const char* const* argv)
     // If we don't have a network config object yet, create it.
     if (c == 0) {
         // Create a new config object.
+        // We don't manage this; the VM will call delete on it.
         c = new TadsNetConfig();
         
         // Get the path to the network config file.
@@ -161,7 +164,9 @@ int main( int argc, char** argv )
         "k:character-set <charset>",
         "l:stat-tcolor <0..7>",
         "n|no-colors",
+#ifdef TADSNET
         "N:net-safety-level <00..44>",
+#endif
         "o|no-defcolors",
         "p|no-pause",
         "R:replay <filename>",
@@ -182,7 +187,7 @@ int main( int argc, char** argv )
     // as the host in a client/server setup (-w option).
     class TadsNetConfig* netconfig = 0;
 
-#ifdef HAVE_SETLOCALE
+#if HAVE_SETLOCALE
     // First, initialise locale, if available. We need only
     // LC_CTYPE.  Don't try to mess with numeric formats!
     setlocale(LC_CTYPE, "");
@@ -362,6 +367,7 @@ int main( int argc, char** argv )
         break;
       }
 
+#ifdef TADSNET
       // --net-safety-level
       case 'N': {
         if (optionError) break;
@@ -393,6 +399,7 @@ int main( int argc, char** argv )
         frobOpts.netSafetyLevelS = tmpS;
         break;
       }
+#endif
 
       // --no-seed-rand
       case 'S': {
