@@ -8,19 +8,26 @@
 #include "common.h"
 #include "osstzprs.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <string.h>
+#include <memory>
+#include <random>
+#include <climits>
+#include <algorithm>
+#include <functional>
+#include <chrono>
+#include <cstdio>
+#include <ctime>
+#include <cstdlib>
+#include <cctype>
+#include <cstring>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <sys/param.h>
-#ifdef HAVE_LANGINFO_CODESET
+#if HAVE_LANGINFO_CODESET
 #include <langinfo.h>
 #endif
-#ifdef HAVE_GLOB_H
+#if HAVE_GLOB_H
 #include <glob.h>
 #endif
 
@@ -469,42 +476,34 @@ os_mkdir( const char* dir, int create_parents )
     // Copy the directory name to a new string so we can strip any trailing
     // path seperators.
     size_t len = strlen(dir);
-    char* tmp = new char[len + 1];
-    strncpy(tmp, dir, len);
+    const auto tmp = std::make_unique<char[]>(len + 1);
+    strncpy(tmp.get(), dir, len);
     while (tmp[len - 1] == OSPATHCHAR)
         --len;
     tmp[len] = '\0';
 
     // If we're creating intermediate diretories, and the path contains
     // multiple elements, recursively create the parent directories first.
-    if (create_parents and strchr(tmp, OSPATHCHAR) != 0) {
+    if (create_parents and strchr(tmp.get(), OSPATHCHAR) != 0) {
         char par[OSFNMAX];
 
         // Extract the parent path.
-        os_get_path_name(par, sizeof(par), tmp);
+        os_get_path_name(par, sizeof(par), tmp.get());
 
         // If the parent doesn't already exist, create it recursively.
         if (osfacc(par) != 0 and not os_mkdir(par, true)) {
-            delete[] tmp;
             return false;
         }
     }
 
     // Create the directory.
-    int ret =
 #if HAVE_MKDIR
-#   if MKDIR_TAKES_ONE_ARG
-             mkdir(tmp);
-#   else
-             mkdir(tmp, S_IRWXU | S_IRWXG | S_IRWXO);
-#   endif
-#elif HAVE__MKDIR
-             _mkdir(tmp);
+    return mkdir(tmp.get(), S_IRWXU | S_IRWXG | S_IRWXO) == 0;
+#elif HAVE_UNDERSCORE_MKDIR
+    return _mkdir(tmp.get()) == 0;
 #else
 #   error "Neither mkdir() nor _mkdir() is available on this system."
 #endif
-    delete[] tmp;
-    return ret == 0;
 }
 
 /* Remove a directory.
@@ -577,16 +576,15 @@ os_file_stat( const char *fname, int follow_links, os_file_stat_t *s )
     // Paranoia.
     if (grpSize > NGROUPS_MAX or grpSize < 0)
         return false;
-    gid_t* groups = new gid_t[grpSize];
-    if (getgroups(grpSize - 1, groups + 1) < 0) {
-        delete[] groups;
+
+    const auto groups = std::make_unique<gid_t[]>(grpSize);
+    if (getgroups(grpSize - 1, groups.get() + 1) < 0) {
         return false;
     }
     groups[0] = getegid();
     int i;
     for (i = 0; i < grpSize and buf.st_gid != groups[i]; ++i)
         ;
-    delete[] groups;
     if (i < grpSize) {
         if (buf.st_mode & S_IRGRP)
             s->attrs |= OSFATTR_READ;
@@ -629,23 +627,12 @@ os_resolve_symlink( const char *fname, char *target, size_t target_size )
 
 
 /* Get a suitable seed for a random number generator.
- *
- * We don't just use the system-clock, but build a number as random as
- * the mechanisms of the standard C-library allow.  This is somewhat of
- * an overkill, but it's simple enough so here goes.  Someone has to
- * write a nuclear war simulator in TADS to test this thoroughly.
  */
 void
 os_rand( long* val )
 {
-    //assert(val != 0);
-    // Use the current time as the first seed.
-    srand(static_cast<unsigned int>(time(0)));
-
-    // Create the final seed by generating a random number using
-    // high-order bits, because on some systems the low-order bits
-    // aren't very random.
-    *val = 1 + static_cast<long>(static_cast<long double>(65535) * rand() / (RAND_MAX + 1.0));
+    static std::default_random_engine rdev(time(0));
+    *val = rdev();
 }
 
 
@@ -656,190 +643,35 @@ os_rand( long* val )
  */
 void os_gen_rand_bytes( unsigned char* buf, size_t len )
 {
-    // Read bytes from /dev/urandom to fill the buffer (use /dev/urandom
-    // rather than /dev/random, so that we don't block for long periods -
-    // /dev/random can be quite slow because it's designed not to return
-    // any bits until a fairly high threshold of entropy has been reached).
-    int f = open("/dev/urandom", O_RDONLY);
-    read(f, (void*)buf, len);
-    close(f);
+    std::independent_bits_engine
+            <std::random_device, CHAR_BIT, unsigned char> eng;
+    std::generate(buf, buf + len, std::ref(eng));
 }
 
 
 /* Get the current system high-precision timer.
- *
- * We provide four (4) implementations of this function:
- *
- *   1. clock_gettime() is available
- *   2. gettimeofday() is available
- *   3. ftime() is available
- *   4. Neither is available - fall back to time()
- *
- * Note that HAVE_CLOCK_GETTIME, HAVE_GETTIMEOFDAY and HAVE_FTIME are
- * mutually exclusive; if one of them is defined, the others aren't.  No
- * need for #else here.
- *
- * Although not required by the TADS VM, these implementations will
- * always return 0 on the first call.
  */
-#ifdef HAVE_CLOCK_GETTIME
-// The system has the clock_gettime() function.
 long
 os_get_sys_clock_ms( void )
 {
-    // We need to remember the exact time this function has been
-    // called for the first time, and use that time as our
-    // zero-point.  On each call, we simply return the difference
-    // in milliseconds between the current time and our zero point.
-    static struct timespec zeroPoint;
-
-    // Did we get the zero point yet?
-    static bool initialized = false;
-
-    // Not all systems provide a monotonic clock; check if it's
-    // available before falling back to the global system-clock.  A
-    // monotonic clock is guaranteed not to change while the system
-    // is running, so we prefer it over the global clock.
-    static const clockid_t clockType =
-#ifdef HAVE_CLOCK_MONOTONIC
-        CLOCK_MONOTONIC;
-#else
-        CLOCK_REALTIME;
-#endif
-    // We must get the current time in each call.
-    struct timespec currTime;
-
-    // Initialize our zero-point, if not already done so.
-    if (not initialized) {
-        clock_gettime(clockType, &zeroPoint);
-        initialized = true;
-    }
-
-    // Get the current time.
-    clock_gettime(clockType, &currTime);
-
-    // Note that tv_nsec contains *nano*seconds, not milliseconds,
-    // so we need to convert it; a millisec is 1.000.000 nanosecs.
-    return (currTime.tv_sec - zeroPoint.tv_sec) * 1000
-         + (currTime.tv_nsec - zeroPoint.tv_nsec) / 1000000;
+    using namespace std::chrono;
+    static const auto start = steady_clock::now();
+    return duration_cast<milliseconds>(steady_clock::now() - start).count();
 }
 
+
 /* Get the time since the Unix Epoch in seconds and nanoseconds.
  */
 void
-os_time_ns( os_time_t *seconds, long *nanoseconds )
+os_time_ns( os_time_t* sec, long* ns )
 {
-    // Get the current time.
-    static const clockid_t clockType = CLOCK_REALTIME;
-    struct timespec currTime;
-    clock_gettime(clockType, &currTime);
-
-    // return the data
-    *seconds = currTime.tv_sec;
-    *nanoseconds = currTime.tv_nsec;
+    using namespace std::chrono;
+    // C++ does not guarantee that the system clock's epoch is the same as
+    // the Unix Epoch, but it's the de-facto standard in all compilers.
+    const auto now = system_clock::now().time_since_epoch();
+    *sec = duration_cast<seconds>(now).count();
+    *ns = duration_cast<nanoseconds>(now - seconds(*sec)).count();
 }
-#endif // HAVE_CLOCK_GETTIME
-
-#ifdef HAVE_GETTIMEOFDAY
-// The system has the gettimeofday() function.
-long
-os_get_sys_clock_ms( void )
-{
-    static struct timeval zeroPoint;
-    static bool initialized = false;
-    // gettimeofday() needs the timezone as a second argument.  This
-    // is obsolete today, but we don't care anyway; we just pass
-    // something.
-    struct timezone bogus = {0, 0};
-    struct timeval currTime;
-
-    if (not initialized) {
-        gettimeofday(&zeroPoint, &bogus);
-        initialized = true;
-    }
-
-    gettimeofday(&currTime, &bogus);
-
-    // 'tv_usec' contains *micro*seconds, not milliseconds.  A
-    // millisec is 1.000 microsecs.
-    return (currTime.tv_sec - zeroPoint.tv_sec) * 1000 + (currTime.tv_usec - zeroPoint.tv_usec) / 1000;
-}
-
-/* Get the time since the Unix Epoch in seconds and nanoseconds.
- */
-void
-os_time_ns( os_time_t *seconds, long *nanoseconds )
-{
-    // get the time
-    struct timezone bogus = {0, 0};
-    struct timeval currTime;
-    gettimeofday(&currTime, &bogus);
-
-    // return the data, converting milliseconds to nanoseconds
-    *seconds = currTime.tv_sec;
-    *nanoseconds = currTime.tv_usec * 1000;
-}
-#endif // HAVE_GETTIMEOFDAY
-
-
-#ifdef HAVE_FTIME
-// The system has the ftime() function.  ftime() is in <sys/timeb.h>.
-#include <sys/timeb.h>
-long
-os_get_sys_clock_ms( void )
-{
-    static timeb zeroPoint;
-    static bool initialized = false;
-    struct timeb currTime;
-
-    if (not initialized) {
-        ftime(&zeroPoint);
-        initialized = true;
-    }
-
-    ftime(&currTime);
-    return (currTime.time - zeroPoint.time) * 1000 + (currTime.millitm - zeroPoint.millitm);
-}
-
-/* Get the time since the Unix Epoch in seconds and nanoseconds.
- */
-void
-os_time_ns( os_time_t *seconds, long *nanoseconds )
-{
-    // get the time
-    struct timeb currTime;
-    ftime(&currTime);
-
-    // return the data, converting milliseconds to nanoseconds
-    *seconds = currTime.time;
-    *nanoseconds = (long)currTime.millitm * 1000000;
-}
-#endif // HAVE_FTIME
-
-
-#ifndef HAVE_CLOCK_GETTIME
-#ifndef HAVE_GETTIMEOFDAY
-#ifndef HAVE_FTIME
-// Use the standard time() function from the C library.  We lose
-// millisecond-precision, but that's the best we can do with time().
-long
-os_get_sys_clock_ms( void )
-{
-    static time_t zeroPoint = time(0);
-    return (time(0) - zeroPoint) * 1000;
-}
-
-void
-os_time_ns( os_time_t *seconds, long *nanoseconds )
-{
-    // get the regular time() value in seconds; we have no
-    // higher precision element to return, so return zero
-    *seconds = time();
-    *nanoseconds = 0;
-}
-#endif
-#endif
-#endif
 
 
 /* Get filename from startup parameter, if possible.
@@ -997,58 +829,56 @@ resolve_path( char *buf, size_t buflen, const char *path )
 
     // make a writable copy of the path to work with
     size_t pathl = strlen(path);
-    char *mypath = new char[pathl + 1];
-    memcpy(mypath, path, pathl + 1);
+    const auto mypath = std::make_unique<char[]>(pathl + 1);
+    memcpy(mypath.get(), path, pathl + 1);
 
     // start at the very end of the path, with no stripped suffix yet
-    char *suffix = mypath + pathl;
+    char *suffix = mypath.get() + pathl;
     char sl = '\0';
 
     // keep going until we resolve something or run out of path
     for (;;)
     {
         // resolve the current prefix, allocating the result
-        char *rpath = realpath(mypath, 0);
+        const std::unique_ptr<char, decltype(&free)>
+                rpath(realpath(mypath.get(), 0), &free);
 
         // un-split the path
         *suffix = sl;
 
         // if we resolved the prefix, return the result
-        if (rpath != 0)
+        if (rpath)
         {
             // success - if we separated a suffix, reattach it
             if (*suffix != '\0')
             {
                 // reattach the suffix (the part after the '/')
                 for ( ; *suffix == '/' ; ++suffix) ;
-                os_build_full_path(buf, buflen, rpath, suffix);
+                os_build_full_path(buf, buflen, rpath.get(), suffix);
             }
             else
             {
                 // no suffix, so we resolved the entire path
-                safe_strcpy(buf, buflen, rpath);
+                safe_strcpy(buf, buflen, rpath.get());
             }
 
-            // done with the resolved path
-            free(rpath);
-
-            // ...and done searching
+            // done searching
             break;
         }
 
         // no luck with realpath(); search for the '/' at the end of the
         // previous component in the path 
-        for ( ; suffix > mypath && *(suffix-1) != '/' ; --suffix) ;
+        for ( ; suffix > mypath.get() && *(suffix-1) != '/' ; --suffix) ;
 
         // skip any redundant slashes
-        for ( ; suffix > mypath && *(suffix-1) == '/' ; --suffix) ;
+        for ( ; suffix > mypath.get() && *(suffix-1) == '/' ; --suffix) ;
 
         // if we're at the root element, we're out of path elements
-        if (suffix == mypath)
+        if (suffix == mypath.get())
         {
             // we can't resolve any part of the path, so just return the
             // original path unchanged
-            safe_strcpy(buf, buflen, mypath);
+            safe_strcpy(buf, buflen, mypath.get());
             break;
         }
 
@@ -1056,9 +886,6 @@ resolve_path( char *buf, size_t buflen, const char *path )
         sl = *suffix;
         *suffix = '\0';
     }
-
-    // done with our writable copy of the path
-    delete [] mypath;
 }
 
 /* Is the given file in the given directory?
@@ -1183,20 +1010,19 @@ os_get_abs_filename( char* buf, size_t buflen, const char* filename )
 
     // Try getting the canonical path from the OS (allocating the
     // result buffer).
-    char* newpath = realpath(filename, 0);
-    if (newpath != 0) {
+    const std::unique_ptr<char, decltype(&free)>
+            newpath(realpath(filename, nullptr), &free);
+    if (newpath) {
         // Copy the output (truncating if it's too long).
-        safe_strcpy(buf, buflen, newpath);
-        free(newpath);
-        return true;
+        safe_strcpy(buf, buflen, newpath.get());
     }
 
-    // realpath() failed, but that's okay - realpath() only works if the
-    // path refers to an existing file, but it's valid for callers to
-    // pass non-existent filenames, such as names of files they're about
-    // to create, or hypothetical paths being used for comparison
-    // purposes or for future use.  Simply return the canonical path
-    // name we generated above.
+    // realpath() might have failed, but that's okay - realpath() only works
+    // if the path refers to an existing file, but it's valid for callers to
+    // pass non-existent filenames, such as names of files they're about to
+    // create, or hypothetical paths being used for comparison purposes or
+    // for future use.  Simply return the canonical path name we generated
+    // above.
     return true;
 }
 
@@ -1426,23 +1252,26 @@ os_get_charmap( char* mapname, int charmap_id )
  * Note that this function is used only by Tads 2.  Tads 3 does mappings
  * automatically.
  *
- * We omit this implementation when not compiling the interpreter (in
- * which case os_xlat_html4 will have been defined as an empty macro in
- * osfrobtads.h).  We do this because we don't want to introduce any
- * TADS 3 dependencies upon the TADS 2 compiler, which should compile
- * regardless of whether the TADS 3 sources are available or not.
+ * We omit an implementation when not compiling the interpreter.   We do this
+ * because we don't want to introduce any TADS 3 dependencies in the TADS 2
+ * compiler, which should compile regardless of whether we're building TADS 3
+ * or not.
  */
-#ifndef os_xlat_html4
 void
 os_xlat_html4( unsigned int html4_char, char* result, size_t result_buf_len )
 {
+#ifdef RUNTIME
     // HTML 4 characters are Unicode.  Tads 3 provides just the
     // right mapper: Unicode to ASCII.  We make it static in order
     // not to create a mapper on each call and save CPU cycles.
     static CCharmapToLocalASCII mapper;
     result[mapper.map_char(html4_char, result, result_buf_len)] = '\0';
-}
+#else
+    (void)html4_char;
+    (void)result;
+    (void)result_buf_len;
 #endif
+}
 
 
 /* =====================================================================
